-- 2023-11-25 10:30
-- v1.3.3

use tuoheng_freeway;

-- 新建周期任务表
create table th_inspection_cycle
(
    id               varchar(36)      default ''                not null comment 'ID' primary key,
    tenant_id        varchar(36)      default ''               not null comment '租户ID',
    dept_id          varchar(36)      default ''                not null comment '部门ID',
    code             varchar(255)     default ''                null comment '任务编号',
    name             varchar(255)     default ''                null comment '任务名称',
    road_id          varchar(36)     default '0'               null comment '巡检公路ID',
    section_id          varchar(36)     default '0'               null comment '巡检路段ID',
    airport_task_id  int unsigned     default '0'               not null comment '机场任务ID',
    airport_id       int unsigned     default '0'               null comment '巡检机场id',
    airport_line_id  int unsigned     default '0'               null comment '巡检线路id',
    drone_id         int unsigned     default '0'               not null comment '执飞无人机ID',
    type             tinyint                                    null comment '执行类型：1,单次;2,每天;5,每周;6,每月',
    single_time      datetime                                   null comment '单次执行时间',
    everyday_time    time                                       null comment '每天执行时间',
    cycle_time       datetime                                   null comment '循环执行时间',
    cycle_next_time  datetime                                   null comment '循环下次执行时间',
    cycle_frequency  varchar(255)                               null comment '循环频率：1~7,每周；1~31,每月',
    cycle_start_time datetime                                   null comment '循环开始时间',
    cycle_end_time   datetime                                   null comment '循环结束时间',
    is_enable        tinyint(1)       default 1                 null comment '是否启用: 0：禁用; 1：启用(默认);',
    note             varchar(255)     default ''                null comment '备注',
    create_user      varchar(36)      default '0'               null comment '创建人',
    create_time      datetime         default CURRENT_TIMESTAMP null comment '创建时间',
    update_user      varchar(36)      default '0'               null comment '更新人',
    update_time      datetime                                   null on update CURRENT_TIMESTAMP comment '更新时间',
    mark             tinyint unsigned default '1'               null comment '有效标记：0删除 1启用 2禁用'
) comment '周期任务表' collate = utf8mb4_bin;

-- 任务表
alter table tuoheng_freeway.th_inspection add inspection_cycle_id varchar(36) default '' not null comment '周期性任务ID' after inspection_type;
alter table tuoheng_freeway.th_inspection add airport_task_id int default 0 not null comment '机场任务ID' after inspection_cycle_id;
alter table tuoheng_freeway.th_inspection add flytime varchar(50) default '' not null comment '总飞行时间（秒）' after fly_height;
alter table tuoheng_freeway.th_inspection add mileage varchar(50) default '' not null comment '总飞行里程（米）' after flytime;

alter table tuoheng_freeway.th_inspection drop column road_name;
alter table tuoheng_freeway.th_inspection drop column section_name;
alter table tuoheng_freeway.th_inspection drop column airport_name;
alter table tuoheng_freeway.th_inspection drop column inspection_line_name;

--周期任务表
alter table tuoheng_freeway.th_inspection_cycle add airport_line_length double(16, 2) null comment '航线长度（公里）' after airport_line_id;

-- 云盒遥测数据表
alter table tuoheng_freeway.th_flight_data add flytime varchar(150) default '' not null comment '总飞行时间：单位秒' after dist_home;
alter table tuoheng_freeway.th_flight_data add mileage varchar(150) default '' not null comment '总里程' after flytime;