package com.tuoheng.admin.entity;

import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tuoheng.common.core.common.BaseEntity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 周期任务表
 *
 * @author wanjing
 * @team tuoheng
 * @date 2023-08-23
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@TableName("th_inspection_cycle")
public class InspectionCycle extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /**
     * 租户ID
     */
    private String tenantId;

    /**
     * 部门ID
     */
    private String deptId;

    /**
     * 任务编号
     */
    private String code;

    /**
     * 任务名称
     */
    private String name;

    /**
     * 巡检公路id
     */
    private String roadId;

    /**
     * 巡检路段id
     */
    private String sectionId;

    /**
     * 巡检机场id
     */
    private Integer airportId;

    /**
     * 航线id
     */
    private Integer airportLineId;

    /**
     * 航线长度
     */
    private Double airportLineLength;

    /**
     * 巡检无人机id
     */
    private Integer droneId;

    /**
     * 执行类型：1,单次;2,每天; 5,每周;6,每月
     */
    private Integer type;

    /**
     * 单次执行时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private Date singleTime;

    /**
     * 每天执行时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private Date everydayTime;

    /**
     * 循环执行时间
     */
    @DateTimeFormat(pattern = "HH:mm:ss")
    @JsonFormat(pattern = "HH:mm:ss", timezone = "GMT+8")
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private Date cycleTime;

    /**
     * 循环下次执行时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private Date cycleNextTime;

    /**
     * 循环频率：1~7,每周；1~31,每月;多个以引文逗号分隔
     */
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private String cycleFrequency;

    /**
     * 循环开始时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private Date cycleStartTime;

    /**
     * 循环结束时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @TableField(updateStrategy = FieldStrategy.IGNORED)   // 添加忽略判断，该字段值不论是什么，都进行更新
    private Date cycleEndTime;

    /**
     * 是否启用: 0：禁用; 1：启用(默认);
     */
    private Integer isEnable;

    /**
     * 备注
     */
    private String note;

    /**
     * 创建人名称
     */
    @TableField(exist = false)
    private String createUserName;

}
