package com.tuoheng.miniprogram.service.wx.bind;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tuoheng.common.core.config.common.CommonConfig;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import com.tuoheng.miniprogram.dao.UserAuthorizeMapper;
import com.tuoheng.miniprogram.dao.UserMapper;
import com.tuoheng.miniprogram.entity.User;
import com.tuoheng.miniprogram.entity.UserAuthorize;
import com.tuoheng.miniprogram.enums.AuthorizeStatusEnum;
import com.tuoheng.miniprogram.enums.MarkEnum;
import com.tuoheng.miniprogram.utils.CurrentUserUtil;
import com.tuoheng.miniprogram.utils.GetOpenIdUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class WeiXinBindService {

    @Autowired
    private UserAuthorizeMapper userAuthorizeMapper;

    @Autowired
    private UserMapper userMapper;


    public JsonResult bind(String code) {
        log.info("进入获取openId接口, code={}", code);
        if (StringUtils.isEmpty(code)) {
            log.info("code为空");
            return JsonResult.error("code为空");
        }
        User user = CurrentUserUtil.getUserInfo();
        String userId = user.getId();
        String openidResult = GetOpenIdUtil.getopenid(code, CommonConfig.appId, CommonConfig.appSecret);
        if (null != JSONObject.parseObject(openidResult).getInteger("errcode") ||
                StringUtils.isNotEmpty(JSONObject.parseObject(openidResult).getString("errmsg"))) {
            return JsonResult.error(JSONObject.parseObject(openidResult).getInteger("errcode").intValue(),
                    JSONObject.parseObject(openidResult).getString("errmsg"));
        }
        String openid = JSONObject.parseObject(openidResult).getString("openid");
        if (StringUtils.isEmpty(openid)) {
            log.error("获取openid失败：", openidResult);
            return JsonResult.error("获取openid失败");
        }

        // 根据当前登录用户id查询授权表是否存在一条授权数据
        UserAuthorize userAuthorize = userAuthorizeMapper.selectOne(Wrappers.<UserAuthorize>lambdaQuery()
                .eq(UserAuthorize::getUserId, userId)
                .eq(UserAuthorize::getOpenId, openid)
                .eq(UserAuthorize::getMark, MarkEnum.VALID.getCode()));
        if (ObjectUtil.isNotEmpty(userAuthorize)) {
            log.info("该用户已做过授权, openid={}", openid);
            return JsonResult.error("该用户已做过授权");
        }

        // 先解绑
        this.unBind(user, openid);

        UserAuthorize userAuthorizeNew = new UserAuthorize();
        userAuthorizeNew.setOpenId(openid);
        userAuthorizeNew.setUserId(userId);
        userAuthorizeNew.setCreateTime(DateUtils.now());
        userAuthorizeNew.setCreateUser(userId);
        int result = userAuthorizeMapper.insert(userAuthorizeNew);
        if (result <= 0) {
            return JsonResult.error("授权数据入库失败");
        }

        //关联登录用户,已经做过授权 修改授权标识
        user.setAuthorize(AuthorizeStatusEnum.AUTHORIZE_ON.getCode());
        user.setUpdateTime(DateUtils.now());
        user.setUpdateUser(userId);
        int count = userMapper.updateById(user);
        if (count <= 0) {
            log.info("用户授权标识更新失败, userId={}", userId);
            return JsonResult.error("用户授权标识更新失败");
        }
        return JsonResult.success();
    }

    private void unBind(User user, String openId) {
        // 当前用户已经与其他微信做过绑定，先解绑
        UserAuthorize userAuthorizeTmp = userAuthorizeMapper.selectOne(Wrappers.<UserAuthorize>lambdaQuery()
                .ne(UserAuthorize::getUserId, user.getId())
                .eq(UserAuthorize::getOpenId, openId)
                .eq(UserAuthorize::getMark, MarkEnum.VALID.getCode()));
        if (ObjectUtil.isNotEmpty(userAuthorizeTmp)) {
            log.info("该openId已与其他用户绑定，先进行解绑, userId={}，openid={}", user.getId(), openId);
            userAuthorizeMapper.deleteById(userAuthorizeTmp.getId());
        }

        userAuthorizeTmp = userAuthorizeMapper.selectOne(Wrappers.<UserAuthorize>lambdaQuery()
                .eq(UserAuthorize::getUserId, user.getId())
                .ne(UserAuthorize::getOpenId, openId)
                .eq(UserAuthorize::getMark, MarkEnum.VALID.getCode()));
        if (ObjectUtil.isNotEmpty(userAuthorizeTmp)) {
            log.info("该用户已绑定，先进行解绑, userId={}，openid={}", user.getId(), openId);
            userAuthorizeMapper.deleteById(userAuthorizeTmp.getId());
        }

        user.setAuthorize(AuthorizeStatusEnum.AUTHORIZE_DOWN.getCode());
        user.setUpdateTime(DateUtils.now());
        user.setUpdateUser(user.getId());
        userMapper.updateById(user);
    }
}
