package com.tuoheng.miniprogram.service.wx.check;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tuoheng.common.core.config.common.CommonConfig;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import com.tuoheng.miniprogram.dao.UserAuthorizeMapper;
import com.tuoheng.miniprogram.dao.UserMapper;
import com.tuoheng.miniprogram.entity.User;
import com.tuoheng.miniprogram.entity.UserAuthorize;
import com.tuoheng.miniprogram.enums.MarkEnum;
import com.tuoheng.miniprogram.utils.CurrentUserUtil;
import com.tuoheng.miniprogram.utils.GetOpenIdUtil;
import com.tuoheng.miniprogram.vo.WxAuthorizeVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class WeiXinCheckAuthorizeService {

    @Autowired
    private UserAuthorizeMapper userAuthorizeMapper;

    public JsonResult checkAuthorize(String code) {
        log.info("进入判断用户是否授权接口, code={}", code);
        User user = CurrentUserUtil.getUserInfo();
        String openidResult = GetOpenIdUtil.getopenid(code, CommonConfig.appId, CommonConfig.appSecret);
        if (null != JSONObject.parseObject(openidResult).getInteger("errcode") || StringUtils.isNotEmpty(JSONObject.parseObject(openidResult).getString("errmsg"))) {
            return JsonResult.error(JSONObject.parseObject(openidResult).getInteger("errcode").intValue(), JSONObject.parseObject(openidResult).getString("errmsg"));
        }
        String openid = JSONObject.parseObject(openidResult).getString("openid");
        if (StringUtils.isEmpty(openid)) {
            log.error("获取openid失败：", openidResult);
            return JsonResult.error("获取openid失败");
        }
        // 根据当前登录用户id和vx查询授权表是否存在一条授权数据
        UserAuthorize beforeUserAuthorize = userAuthorizeMapper.selectOne(Wrappers.<UserAuthorize>lambdaQuery()
                .eq(UserAuthorize::getUserId, user.getId())
                .eq(UserAuthorize::getOpenId, openid)
                .eq(UserAuthorize::getMark, MarkEnum.VALID.getCode()));
        WxAuthorizeVo wxAuthorizeVo = new WxAuthorizeVo();
        wxAuthorizeVo.setOpenId(openid);
        if (ObjectUtil.isNotEmpty(beforeUserAuthorize)) {
            wxAuthorizeVo.setIsAuthroize(true);
        } else {
            wxAuthorizeVo.setIsAuthroize(false);
        }
        return JsonResult.success(wxAuthorizeVo);
    }
}
